
/*
* Licensed to the Apache Software Foundation (ASF) under one
* or more contributor license agreements.  See the NOTICE file
* distributed with this work for additional information
* regarding copyright ownership.  The ASF licenses this file
* to you under the Apache License, Version 2.0 (the
* "License"); you may not use this file except in compliance
* with the License.  You may obtain a copy of the License at
*
*   http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing,
* software distributed under the License is distributed on an
* "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
* KIND, either express or implied.  See the License for the
* specific language governing permissions and limitations
* under the License.
*/



/**
 * AUTO-GENERATED FILE. DO NOT MODIFY.
 */

import { SOURCE_FORMAT_KEYED_COLUMNS } from '../../util/types';
import { makePrintable, throwError } from '../../util/log';
import { isArray, each, hasOwn } from 'zrender/esm/core/util';
import { normalizeToArray } from '../../util/model';
import { parseDate } from '../../util/number';
var SortOrderValidMap = {
  asc: true,
  desc: true
};
var sampleLog = '';

if (process.env.NODE_ENV !== 'production') {
  sampleLog = ['Valid config is like:', '{ dimension: "age", order: "asc" }', 'or [{ dimension: "age", order: "asc"], { dimension: "date", order: "desc" }]'].join('');
}

var timeParser = function (val) {
  return +parseDate(val);
};

export var sortTransform = {
  type: 'echarts:sort',
  transform: function transform(params) {
    var source = params.source;
    var config = params.config;
    var errMsg = '';
    var orderExprList = normalizeToArray(config);

    if (!orderExprList.length) {
      if (process.env.NODE_ENV !== 'production') {
        errMsg = 'Empty `config` in sort transform.';
      }

      throwError(errMsg);
    }

    var orderDefList = [];
    each(orderExprList, function (orderExpr) {
      var dimLoose = orderExpr.dimension;
      var order = orderExpr.order;
      var parserName = orderExpr.parse;

      if (dimLoose == null) {
        if (process.env.NODE_ENV !== 'production') {
          errMsg = 'Sort transform config must has "dimension" specified.' + sampleLog;
        }

        throwError(errMsg);
      }

      if (!hasOwn(SortOrderValidMap, order)) {
        if (process.env.NODE_ENV !== 'production') {
          errMsg = 'Sort transform config must has "order" specified.' + sampleLog;
        }

        throwError(errMsg);
      }

      var dimInfo = source.getDimensionInfo(dimLoose);

      if (!dimInfo) {
        if (process.env.NODE_ENV !== 'production') {
          errMsg = makePrintable('Can not find dimension info via: "' + dimLoose + '".\n', 'Existing dimensions: ', source.getDimensionInfoAll(), '.\n', 'Illegal config:', orderExpr, '.\n');
        }

        throwError(errMsg);
      }

      var parser;

      if (parserName) {
        if (parserName !== 'time') {
          if (process.env.NODE_ENV !== 'production') {
            errMsg = makePrintable('Invalid parser name' + parserName + '.\n', 'Illegal config:', orderExpr, '.\n');
          }

          throwError(errMsg);
        }

        parser = timeParser;
      }

      orderDefList.push({
        dimIdx: dimInfo.index,
        orderReturn: order === 'asc' ? -1 : 1,
        parser: parser
      });
    });

    if (!isArray(source.data)) {
      if (process.env.NODE_ENV !== 'production') {
        errMsg = source.sourceFormat === SOURCE_FORMAT_KEYED_COLUMNS ? 'sourceFormat ' + SOURCE_FORMAT_KEYED_COLUMNS + ' is not supported yet' : source.data == null ? 'Upstream source data is null/undefined' : 'Unsupported source format.';
      }

      throwError(errMsg);
    }

    var sourceHeaderCount = source.sourceHeaderCount;
    var resultData = [];
    var headerPlaceholder = {};

    for (var i = 0; i < sourceHeaderCount; i++) {
      resultData.push(headerPlaceholder);
    }

    for (var i = 0, len = source.count(); i < len; i++) {
      resultData.push(source.getRawDataItem(i));
    }

    resultData.sort(function (item0, item1) {
      if (item0 === headerPlaceholder) {
        return -1;
      }

      if (item1 === headerPlaceholder) {
        return 1;
      }

      if (item0 == null) {
        return 1;
      }

      if (item1 == null) {
        return -1;
      }

      for (var i = 0; i < orderDefList.length; i++) {
        var orderDef = orderDefList[i];
        var val0 = source.retrieveItemValue(item0, orderDef.dimIdx);
        var val1 = source.retrieveItemValue(item1, orderDef.dimIdx);

        if (orderDef.parser) {
          val0 = orderDef.parser(val0);
          val1 = orderDef.parser(val1);
        }

        if (val0 < val1) {
          return orderDef.orderReturn;
        } else if (val0 > val1) {
          return -orderDef.orderReturn;
        }
      }

      return 0;
    });

    for (var i = 0; i < sourceHeaderCount; i++) {
      resultData[i] = source.getRawHeaderItem(i);
    }

    return {
      data: resultData
    };
  }
};